@@ -1,8 +1,8 @@
 //
 // This file is part of the aMule Project.
 //
-// Copyright (c) 2003-2005 aMule Team ( admin@amule.org / http://www.amule.org )
-// Copyright (c) 2003-2005 Madcat ( madcat@_@users.sf.net / sharedaemon.sf.net )
+// Copyright (c) 2003-2009 aMule Team ( admin@amule.org / http://www.amule.org )
+// Copyright (c) 2003-2009 Madcat ( madcat@_@users.sf.net / sharedaemon.sf.net )
 //
 // Any parts of this program derived from the xMule, lMule or eMule project,
 // or contributed by third-party developers are copyrighted by their
@@ -20,22 +20,104 @@
 // 
 // You should have received a copy of the GNU General Public License
 // along with this program; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA, 02111-1307, USA
+// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301, USA
 //
 
 const int versionMajor		= 1;
-const int versionMinor		= 1;
-const int versionRevision	= 0;
+const int versionMinor		= 4;
+const int versionRevision	= 1;
 
-
-#include <string>
+#include <cstdlib>
 #include <sstream>
 #include <iostream>
 #include <fstream>
 
+#ifdef __APPLE__
+	#include <CoreServices/CoreServices.h>
+#elif defined(_WIN32)
+	#include <winerror.h>
+	#include <shlobj.h>
+	#include <shlwapi.h>
+#endif
+
+#include "FileLock.h"
+#include "MagnetURI.h"
+#include "MuleCollection.h"
 
 using std::string;
 
+string GetLinksFilePath(const string& configDir)
+{
+	if (!configDir.empty()) {
+#ifdef _WIN32
+		char buffer[MAX_PATH + 1];
+		configDir.copy(buffer, MAX_PATH);
+		if (PathAppendA(buffer, "ED2KLinks")) {
+			string strDir;
+			strDir.assign(buffer);
+			return strDir;
+		}
+#else
+		string strDir = configDir;
+		if (strDir.at(strDir.length() - 1) != '/') {
+			strDir += '/';
+		}
+		return strDir + "ED2KLinks";
+#endif
+	}
+
+
+#ifdef __APPLE__
+
+	std::string strDir;
+	
+	FSRef fsRef;
+	if (FSFindFolder(kUserDomain, kApplicationSupportFolderType, kCreateFolder, &fsRef) == noErr) {
+		CFURLRef urlRef = CFURLCreateFromFSRef(NULL, &fsRef);
+		if (urlRef != NULL) {
+			UInt8 buffer[PATH_MAX + 1];
+			if (CFURLGetFileSystemRepresentation(urlRef, true, buffer, sizeof(buffer))) {
+				strDir.assign((char*) buffer);
+			}
+			CFRelease(urlRef) ;
+		}
+	}
+
+	return strDir + "/aMule/ED2KLinks";
+
+#elif defined(_WIN32)
+
+	std::string strDir;
+	LPITEMIDLIST pidl;
+	char buffer[MAX_PATH + 1];
+
+	HRESULT hr = SHGetSpecialFolderLocation(NULL, CSIDL_APPDATA, &pidl);
+
+	if (SUCCEEDED(hr)) {
+		if (SHGetPathFromIDListA(pidl, buffer)) {
+			if (PathAppendA(buffer, "aMule\\ED2KLinks")) {
+				strDir.assign(buffer);
+			}
+		}
+	}
+
+	if (pidl) {
+		LPMALLOC pMalloc;
+		SHGetMalloc(&pMalloc);
+		if (pMalloc) {
+			pMalloc->Free(pidl);
+			pMalloc->Release();
+		}
+	}
+
+	return strDir;
+
+#else
+
+	return string( getenv("HOME") ) + "/.aMule/ED2KLinks";
+
+#endif
+}
 
 /**
  * Converts a hexadecimal number to a char.
@@ -184,18 +266,17 @@ void badLink( const string& type, const string& err, const string& uri )
  *
  * If errors are detected, it will terminate the program.
  */
-void writeLink( const string& uri )
+void writeLink( const string& uri, const string& config_dir )
 {
-	// ofstream is static in order to avoid opening/closing the file for each
-	// link found i the parameters.
+	// Attempt to lock the ED2KLinks file
+	static CFileLock lock(GetLinksFilePath(config_dir));
 	static std::ofstream file;
 	
-	if ( !file.is_open() ) {
-		string path = string( getenv("HOME") ) + "/.aMule/ED2KLinks";
-		
+	if (!file.is_open()) {
+		string path = GetLinksFilePath(config_dir);
 		file.open( path.c_str(), std::ofstream::out | std::ofstream::app );
 
-		if ( !file.is_open() ) {
+		if (!file.is_open()) {
 			std::cout << "ERROR! Failed to open " << path << " for writing!" << std::endl;
 			exit(1);
 		}
@@ -203,7 +284,7 @@ void writeLink( const string& uri )
 
 	file << uri << std::endl;
 
-	std::cout << "Link succesfully queued." << std::endl;
+	std::cout << "Link successfully queued." << std::endl;
 }
 
 
@@ -251,8 +332,6 @@ bool checkFileLink( const string& uri )
 			return false;
 		}
 
-		writeLink( uri );
-
 		return true;
 	}
 
@@ -296,8 +375,6 @@ bool checkServerLink( const string& uri )
 			return false;
 		}
 
-		writeLink( uri );
-
 		return true;
 	}
 
@@ -333,8 +410,6 @@ bool checkServerListLink( const string& uri )
 			return false;
 		}
 
-		writeLink( uri );
-
 		return true;
 	}
 
@@ -345,10 +420,21 @@ bool checkServerListLink( const string& uri )
 int main(int argc, char *argv[])
 {
 	bool errors = false;
-	
+	string config_path;
 	for ( int i = 1; i < argc; i++ ) {
 		string arg = strip( Unescape( string( argv[i] ) ) );
-		
+
+		if ( arg.compare(0, 7, "magnet:" ) == 0 ) {
+			string ed2k = CMagnetED2KConverter(arg);
+			if ( ed2k.empty() ) {
+				std::cerr << "Cannot convert magnet URI to ed2k:\n\t" << arg << std::endl;
+				errors = true;
+				continue;
+			} else {
+				arg = ed2k;
+			}
+		}
+
 		if ( arg.substr( 0, 8 ) == "ed2k://|" ) {
 			// Ensure the URI is valid
 			if ( arg.at( arg.length() - 1 ) != '/' ) {
@@ -357,35 +443,67 @@ int main(int argc, char *argv[])
 			
 			string type = arg.substr( 8, arg.find( '|', 9 ) - 8 );
 		
-			if ( type == "file" ) {
-				errors |= !checkFileLink( arg );
-			} else if ( type == "server" ) {
-				errors |= !checkServerLink( arg );
-			} else if ( type == "serverlist" ) {
-				errors |= !checkServerListLink( arg );
+			if ( (type == "file") && checkFileLink( arg ) ) {
+				writeLink( arg, config_path );
+			} else if ( (type == "server") && checkServerLink( arg ) ) {
+				writeLink( arg, config_path );
+			} else if ( (type == "serverlist") && checkServerListLink( arg ) ) {
+				writeLink( arg, config_path );
 			} else {
-				std::cout << "Unknown link-type:\n\t" << arg << std::endl;
+				std::cout << "Unknown or invalid link-type:\n\t" << arg << std::endl;
+				errors = true;
 			}
-		} else if ( arg == "--help" ) {
+		} else if (arg == "-c" || arg == "--config-dir") {
+			if (i < argc - 1) {
+				config_path = argv[++i];
+			} else {
+				std::cerr << "Missing mandatory argument for " << arg << std::endl;
+				errors = true;
+			}
+		} else if (arg.substr(0, 2) == "-c") {
+			config_path = arg.substr(2);
+		} else if (arg.substr(0, 13) == "--config-dir=") {
+			config_path = arg.substr(13);
+		} else if (arg == "-h" || arg == "--help") {
 			std::cout << getVersion()
 				<< "\n\n"
 				<< "Usage:\n"
-				<< "    --help              Prints this help.\n"
-				<< "    --version           Displays version info.\n\n"
-				<< "    ed2k://|file|       Causes the file to be queued for download.\n"
-				<< "    ed2k://|server|     Causes the server to be listed or updated.\n"
-				<< "    ed2k://|serverlist| Causes aMule to update the current serverlist."
+				<< "    --help, -h              Prints this help.\n"
+				<< "    --config-dir, -c        Specifies the aMule configuration directory.\n"
+				<< "    --version, -v           Displays version info.\n\n"
+				<< "    magnet:?                Causes the file to be queued for download.\n"
+				<< "    ed2k://|file|           Causes the file to be queued for download.\n"
+				<< "    ed2k://|server|         Causes the server to be listed or updated.\n"
+				<< "    ed2k://|serverlist|     Causes aMule to update the current serverlist.\n\n"
+				<< "    --emulecollection, -e   Loads all links of an emulecollection\n\n"
+				<< "*** NOTE: Option order is important! ***\n"
 				<< std::endl;
 			
-		} else if ( arg == "--version" ) {
+		} else if (arg == "-v" || arg == "--version") {
+			std::cout << getVersion() << std::endl;
+		} else if (arg == "-e" || arg == "--emulecollection") {
+			if (i < argc - 1) {
+				CMuleCollection my_collection;
+				if (my_collection.Open( /* emulecollection file */ argv[++i] ))
+				{
+					for(size_t e = 0;e < my_collection.GetFileCount();e++)
+						writeLink( my_collection.GetEd2kLink(e), config_path );
+				} else {
+					std::cerr << "Invalid emulecollection file: " << argv[i] << std::endl;
+					errors = true;
+				}
+			} else {
+				std::cerr << "Missing mandatory argument for " << arg << std::endl;
+				errors = true;
+			}
 			std::cout << getVersion() << std::endl;
 		} else {
-			std::cout << "Bad parameter value:\n\t" << arg << "\n" << std::endl;
+			std::cerr << "Bad parameter value:\n\t" << arg << "\n" << std::endl;
 			errors = true;
 		}
-
 	}
 
 	return ( errors ? 1 : 0 );
 }
 
+// File_checked_for_headers
